/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.validator;

import java.io.IOException;
import java.util.Collection;

/**
 * 委譲先に委譲して委譲した結果を返す{@link ValidatorExpression}。
 * {@link ValidatorContext}に対して状態を変更する操作を行わない。
 * @author nakamura
 *
 */
public class DelegationValidator extends ValidatorExpressionAdapter {
	private final ValidatorExpression expression;
	
	/**
	 * コンストラクタ。
	 * @param expression 委譲先。
	 * @throws NullPointerException 引数が null の場合。
	 */
	public DelegationValidator(final ValidatorExpression expression){
		expression.getClass();
		this.expression = expression;
	}

	@Override public boolean interpret(final ValidatorContext context) {
		return expression.interpret(context);
	}

	/**
	 * JavaScriptコンストラクタツリーを生成するために呼び出される(called)。
	 * @param sb 作成途中の JavaScript コンストラクタツリーの文字列表現。
	 */
	@Override public void appendArgumentTo(final Appendable sb) throws IOException{
		expression.appendTo(sb);
	}

	@Override public void addMessageTo(final Collection<String> messageCollection) {
		expression.addMessageTo(messageCollection);
	}
}
