/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.validator;

import java.io.IOException;
import java.util.Collection;

/**
 * 値検証/値変換の機能を拡張するために使用する インタープリタ デザインパターンの Expression 役。
 * 実装クラスはステートレスであるべき。
 * ただしキャッシュなどステートレスに収束するもの・またはテストで使用するものを除く。
 * @author nakamura
 *
 */
public interface ValidatorExpression {

	/**
	 * 値検証を実行するために呼び出される(called)。
	 * 実装クラスでは引数が null でないことを前提としてよい。
	 * 実装クラスで{@link RuntimeException}のサブ例外が発生する場合は JavaDoc コメントに記載すると部品の可用性が維持される。
	 * @param context Context 役。
	 * @return 値検証結果。
	 */
	public boolean interpret(ValidatorContext context);
	
	/**
	 * JavaScript のコンストラクタツリーを生成するために呼び出される(called)。
	 * 実装クラスでは、引数が null でないことを前提としてよい。
	 * @param sb 作成途中の JavaScript コンストラクタツリーの文字列表現。
	 */
	public void appendTo(Appendable sb) throws IOException;
	
	/**
	 * コンストラクタツリーに定義されている全てのメッセージを収集するために呼び出される(called)。
	 * メッセージを定義する実装クラスでメッセージを追加する。
	 * 収集したメッセージ一覧のそれぞれのメッセージの妥当性を検証するために使用することを想定している。
	 * 実装クラスでは引数が null でないことを前提としてよい。
	 * @param messageCollection メッセージの{@link Collection}。
	 */
	public void addMessageTo(Collection<String> messageCollection);
}
